#include <Wire.h>
#include <Servo.h>
#include <math.h>

// Create servo objects
Servo R2, R1, L2, L1;

#define R2pin 7
#define R1pin 6
#define L1pin 1
#define L2pin 0

float amplitude[4] = {30, 30, 20, 20};
float offset[4]    = {0, 0, 4, -4};
float phase[4]     = {0, 0, -PI/2, -PI/2};
int T = 1000; // 1 gait cycle = 1 sec

unsigned long servoStartTime;
bool walking = false;

void receiveEvent(int numBytes) {
  if (Wire.available()) {
    int x = Wire.read();
    if (x == 1) {
      walking = true;
    } 
    else if (x == 0) {
      walking = false;
    }
    Serial.print("Received: ");
    Serial.println(x);
  }
}

void setup() {
  Wire.swap(1);   // Use alternate I2C pins for ATtiny1614
  Wire.begin(8);  // I2C slave address 8
  Wire.onReceive(receiveEvent);
  Serial.begin(9600);

  R2.attach(R2pin);
  R1.attach(R1pin);
  L1.attach(L1pin);
  L2.attach(L2pin);

  centerServos();
  delay(1000);
}

void loop() {
  if (walking) {
    walkCycle();
  } 
  else {
    centerServos();
  }
}

void walkCycle() {
  unsigned long elapsed = millis() - servoStartTime;
  float t = (float)elapsed / T;
  if (t > 1.0) t = 1.0;

  for (int i = 0; i < 4; i++) {
    float theta = amplitude[i] * sin(2 * PI * t + phase[i]) + offset[i];
    int pos = 90 + theta;
    switch (i) {
      case 0: L1.write(pos); break;
      case 1: R1.write(pos); break;
      case 2: L2.write(pos); break;
      case 3: R2.write(pos); break;
    }
  }

  if (elapsed >= T) {
    servoStartTime = millis();  // immediately start next step
  }
}

void centerServos() {
  R2.write(90);
  R1.write(90);
  L1.write(90);
  L2.write(90);
}